use Admin;
go

drop table if exists tempdb.dbo.EventInfo;
go
create table tempdb.dbo.EventInfo
(
	EventId int not null identity(1,1) primary key clustered,
	EventDate date not null,
	Notes nvarchar(200) null
);
create unique nonclustered index uq_EventInfo__EventDate on tempdb.dbo.EventInfo (EventDate);
go

declare @startDate date = '2017-01-01';
declare @endDate date = getdate();
declare @date1 date = dateadd(day, (2*3*5*71+1) * 3, '2000-01-01');
declare @date2 date = dateadd(day, 3*3*3*5*7*19*41, cast('0001-12-25' as date));
declare @date3 date = dateadd(day, 2*-7*(1+2*2*43), '2024-04-08');
--select @date1, @date2, @date3;

insert tempdb.dbo.EventInfo (EventDate)
select dateadd(day, n.n - 1, @startDate)
from Admin.dbo.fnGetNums(10000) n
where n.n <= datediff(day, @startDate, @endDate) + 1
and dateadd(day, n.n - 1, @startDate) not in (@date1, @date2, @date3);
go

select EventDate
from tempdb.dbo.EventInfo
order by EventDate;
go

-- Determine the offset of the EventDate value relative to the first date in the table
select	ev.EventDate,
		datediff(day, (select min(EventDate) from tempdb.dbo.EventInfo), ev.EventDate) + 1  daynum
from tempdb.dbo.EventInfo ev
order by ev.EventDate;

-- Assign a row_number ordered by the offset (same as saying ordered by EventDate).
-- From this it becomes evident that dates are missing!
with DateNum as
(
	select	ev.EventDate,
			datediff(day, (select min(EventDate) from tempdb.dbo.EventInfo), ev.EventDate) + 1  daynum
	from tempdb.dbo.EventInfo ev
)
select dn.EventDate,
	dn.daynum,
	row_number() over (order by dn.daynum) rn
from DateNum dn
order by dn.EventDate;

-- Compute the difference between the offset and the row_number
with DateNum as
(
	select	ev.EventDate,
			datediff(day, (select min(EventDate) from tempdb.dbo.EventInfo), ev.EventDate) + 1  daynum
	from tempdb.dbo.EventInfo ev
), DateWithRowNum as
(
	select dn.EventDate,
		dn.daynum,
		row_number() over (order by dn.daynum) rn
	from DateNum dn
)
select dwrn.EventDate,
       dwrn.daynum,
       dwrn.rn,
	   dwrn.daynum - dwrn.rn delta
from DateWithRowNum dwrn
order by dwrn.EventDate;

-- Identify ranges (ISLANDS)
with DateNum as
(
	select	ev.EventDate,
			datediff(day, (select min(EventDate) from tempdb.dbo.EventInfo), ev.EventDate) + 1  daynum
	from tempdb.dbo.EventInfo ev
), DateWithRowNum as
(
	select dn.EventDate,
		dn.daynum,
		row_number() over (order by dn.daynum) rn
	from DateNum dn
), DateWithDelta as
(
	select dwrn.EventDate,
		   dwrn.daynum,
		   dwrn.rn,
		   dwrn.daynum - dwrn.rn delta
	from DateWithRowNum dwrn
)
select	dwd.delta,
		min(dwd.EventDate) FirstDateInRange,
		max(dwd.EventDate) LastDateInRange
from DateWithDelta dwd
group by dwd.delta
order by dwd.delta;

-- Find GAPS
with DateNum as
(
	select	ev.EventId,
			ev.EventDate,
			datediff(day, (select min(EventDate) from tempdb.dbo.EventInfo), ev.EventDate) + 1  daynum
	from tempdb.dbo.EventInfo ev
), DateWithRowNum as
(
	select dn.EventDate,
		dn.daynum,
		row_number() over (order by dn.daynum) rn
	from DateNum dn
), DateWithDelta as
(
	select dwrn.EventDate,
		   dwrn.daynum,
		   dwrn.rn,
		   dwrn.daynum - dwrn.rn delta
	from DateWithRowNum dwrn
), Islands as
(
	select	dwd.delta,
			min(dwd.EventDate) FirstDateInRange,
			max(dwd.EventDate) LastDateInRange
	from DateWithDelta dwd
	group by dwd.delta
), GapRanges as
(
	select	delta,
			dateadd(day, 1, lag(i.LastDateInRange) over (order by delta)) GapStartDate,
			dateadd(day, -1, i.FirstDateInRange) GapEndDate
	from Islands i
)
select g.GapStartDate, g.GapEndDate
from GapRanges g
where delta > 0;
