SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO
if exists (select * from sys.objects o inner join sys.schemas s on o.schema_id = s.schema_id
	where s.name = 'dbo' and o.name = 'fnBackupFilesForRestore' and o.type = 'IF')
begin
	drop function dbo.fnBackupFilesForRestore;
end
go

create function dbo.fnBackupFilesForRestore
(
	@restoreTime datetime2,
	@includeDiffs bit = 0
)
returns table as
return
(
	with MostRecentFull as
	(
		select		MappingId, DatabaseName, FirstLSN, LastLSN
		from		
		(
			select		MappingId, DatabaseName, FirstLSN, LastLSN,
						row_number() over (partition by blm.DatabaseName order by blm.BackupEndDate desc) rn
			from		dbo.BackupLsnMapping blm
			where		blm.BackupType = 'Full'
			and			blm.BackupEndDate <= @restoreTime
		) FullBackups
		where		rn = 1
	), LatestDiff as
	(
		select		MappingId, DatabaseName, FirstLSN, LastLSN, BackupEndDate
		from
		(
			select		blm.MappingId, blm.DatabaseName, blm.FirstLSN, blm.LastLSN, blm.BackupEndDate,
						row_number() over (partition by blm.DatabaseName order by blm.BackupEndDate desc) rn
			from		dbo.BackupLsnMapping blm
			inner join	MostRecentFull mrf on blm.DatabaseName = mrf.DatabaseName
			where		blm.BackupType = 'Diff'
			and			blm.DifferentialLSN = mrf.FirstLSN
			and			blm.BackupEndDate <= @restoreTime
		) SubsequentDiffs
		where		rn = 1
		and			@includeDiffs = 1
	), SubsequentLogs as
	(
		select		blm.MappingId, blm.DatabaseName, blm.FirstLSN, blm.BackupEndDate,
					row_number() over (partition by blm.DatabaseName order by blm.FirstLSN, blm.BackupEndDate) rn
		from		dbo.BackupLsnMapping blm
		inner join	MostRecentFull mrf on blm.DatabaseName = mrf.DatabaseName
		left join	LatestDiff ld on blm.DatabaseName = ld.DatabaseName
		where		blm.BackupType = 'Log'
		and			blm.LastLSN >= coalesce(ld.LastLSN, mrf.LastLSN)
		and			(ld.BackupEndDate is null or ld.BackupEndDate <= @restoreTime)
	), FirstLogAfterRestoreTime as
	(
		select		sl.DatabaseName, sl.rn
		from
		(
			select		sl.DatabaseName, sl.rn,
						row_number() over (partition by sl.DatabaseName order by sl.rn) rn2
			from
			(
				select		sl.DatabaseName, sl.BackupEndDate, sl.rn
				from		SubsequentLogs sl

				union all

				-- This is included to guarantee that the FirstLogAfterRestoreTime CTE always returns a row for each database,
				-- even if the restore point time is beyond where the log backups currently are at
				select		distinct sl.DatabaseName, '9999-12-31' BackupEndDate, 2147483647 rn
				from		SubsequentLogs sl
			) sl
			where		sl.BackupEndDate > @restoreTime
		) sl
		where		sl.rn2 = 1
	), LogBackups as
	(
		select		sl.MappingId, sl.DatabaseName, sl.FirstLSN, sl.BackupEndDate
		from		SubsequentLogs sl
		inner join	FirstLogAfterRestoreTime flart on sl.DatabaseName = flart.DatabaseName and sl.rn <= flart.rn
	), RestoreFiles as
	(	
		select MappingId, 1 as RestoreSequence
		from MostRecentFull
		union all
		select MappingId, 2 as RestoreSequence
		from LatestDiff
		union all
		select MappingId, 2 + row_number() over (partition by DatabaseName order by FirstLSN, BackupEndDate) RestoreSequence
		from LogBackups
	)
	select blm.MappingId, blm.ServerName, blm.DatabaseName, blm.BackupType, blm.LocalFileName, blm.SharedFileName,
		blm.FirstLSN, blm.LastLSN, blm.DifferentialLSN, blm.BackupStartDate, blm.BackupEndDate,
		row_number() over (partition by blm.DatabaseName order by rf.RestoreSequence) RestoreSequence
	from RestoreFiles rf
	inner join dbo.BackupLsnMapping blm on rf.MappingId = blm.MappingId
);
GO
